@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/parser/pool_memory.h,v 1.2 2007/01/04 17:27:11 devrim Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/parser/pool_memory.h,v 1.5 2008/01/29 01:56:39 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2007	PgPool Global Development Group
+ * Copyright (c) 2003-2008	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -27,6 +27,9 @@
 
 #define SLOT_NUM 11
 
+#define PARSER_BLOCK_SIZE 8192
+#define PREPARE_BLOCK_SIZE 1024
+
 typedef struct POOL_BLOCK {
 	int size;
 	int allocsize;
@@ -47,6 +50,7 @@ typedef struct POOL_CHUNK {
 
 typedef struct {
 	int size;
+	int blocksize;
 	POOL_BLOCK *blocks;
 	POOL_BLOCK *largeblocks;
 	POOL_CHUNK *freelist[SLOT_NUM];
@@ -57,8 +61,8 @@ extern POOL_MEMORY_POOL *pool_memory;
 extern void *pool_memory_alloc(POOL_MEMORY_POOL *pool, unsigned int size);
 extern void pool_memory_free(POOL_MEMORY_POOL *pool, void *ptr);
 extern void *pool_memory_realloc(POOL_MEMORY_POOL *pool, void *ptr, unsigned int size);
-extern POOL_MEMORY_POOL *pool_memory_create(void);
-extern void pool_memory_delete(POOL_MEMORY_POOL *pool_memory);
+extern POOL_MEMORY_POOL *pool_memory_create(int blocksize);
+extern void pool_memory_delete(POOL_MEMORY_POOL *pool_memory, int reuse);
 extern char *pool_memory_strdup(POOL_MEMORY_POOL *pool_memory, const char *string);
 extern void *pool_memory_alloc_zero(POOL_MEMORY_POOL *pool_memory, unsigned int size);
 
