@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.c,v 1.6.2.7 2009/08/06 07:55:17 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.c,v 1.6.2.13 2009/10/02 07:56:42 t-ishii Exp $
  * 
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -68,8 +68,14 @@ int execute_select = 0; /* non 0 if select query is in transaction */
 
 /* non 0 if "BEGIN" query with extended query protocol received */
 int receive_extended_begin = 0;
-/* non 0 if allow to close internal transaction */
-int allow_close_transaction = 1;
+
+/*
+ * Non 0 if allow to close internal transaction.  This variable was
+ * introduced on 2008/4/3 not to close an internal transaction when
+ * Sync message is received after receiving Parse message. This hack
+ * is for PHP-PDO.
+ */
+static int allow_close_transaction = 1;
 
 PreparedStatementList prepared_list; /* prepared statement name list */
 
@@ -91,7 +97,7 @@ char *parsed_query = NULL;
 static int check_errors(POOL_CONNECTION_POOL *backend, int backend_id);
 static void generate_error_message(char *prefix, int specific_error, char *query);
 
-POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 										POOL_CONNECTION_POOL *backend)
 {
 	int pid, pid1;
@@ -131,7 +137,7 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
  * Process Query('Q') message
  * Query messages include a SQL string.
  */
- POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend, 
+ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 						 POOL_CONNECTION_POOL *backend, char *query)
 {
 	char *string, *string1;
@@ -231,19 +237,19 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 
 		if (pool_config->parallel_mode)
 		{
-      /* The Query is analyzed first in a parallel mode(in_parallel_query), 
+      /* The Query is analyzed first in a parallel mode(in_parallel_query),
        * and, next, the Query is rewritten(rewrite_query_stmt).
        */
- 
+
 			/* analyze the query */
 			RewriteQuery *r_query = is_parallel_query(node,backend);
 
 			if(r_query->is_loadbalance)
 			{
-        /* Usual processing of pgpool is done by using the rewritten Query 
-         * if judged a possible load-balancing as a result of analyzing 
-         * the Query. 
-         * Of course, the load is distributed only for load_balance_mode=true. 
+        /* Usual processing of pgpool is done by using the rewritten Query
+         * if judged a possible load-balancing as a result of analyzing
+         * the Query.
+         * Of course, the load is distributed only for load_balance_mode=true.
          */
 				if(r_query->r_code ==  SEND_LOADBALANCE_ENGINE)
 				{
@@ -255,8 +261,8 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				pool_debug("SimpleQuery: loadbalance_query =%s",string);
 			}
 			else if (r_query->is_parallel)
-			{ 
-				/* 
+			{
+				/*
 				 * For the Query that the parallel processing is possible.
 				 * Call parallel exe engine and return status to the upper layer.
 				 */
@@ -477,14 +483,14 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 		if (REPLICATION)
 		{
 			/* start a transaction if needed */
-			if (start_internal_transaction(backend, (Node *)node) != POOL_CONTINUE)
+			if (start_internal_transaction(frontend, backend, (Node *)node) != POOL_CONTINUE)
 				return POOL_END;
 
 			/* check if need lock */
 			if (need_insert_lock(backend, string, node))
 			{
 				/* if so, issue lock command */
-				status = insert_lock(backend, string, (InsertStmt *)node);
+				status = insert_lock(frontend, backend, string, (InsertStmt *)node);
 				if (status != POOL_CONTINUE)
 				{
 					free_parser();
@@ -492,7 +498,7 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				}
 			}
 		}
-		else if (REPLICATION && query == NULL && start_internal_transaction(backend, node))
+		else if (REPLICATION && query == NULL && start_internal_transaction(frontend, backend, node))
 		{
 			free_parser();
 			return POOL_ERROR;
@@ -506,7 +512,7 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 			master_slave_was_enabled = 1;
 			MASTER_SLAVE = 0;
 			master_slave_dml = 1;
-		}		
+		}
 	}
 
 	if (MAJOR(backend) == PROTO_MAJOR_V2 && is_start_transaction_query(node))
@@ -530,7 +536,7 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 			if (send_simplequery_message(MASTER(backend), len, string, MAJOR(backend)) != POOL_CONTINUE)
 				return POOL_END;
 
-			if (wait_for_query_response(frontend, MASTER(backend), string) != POOL_CONTINUE)
+			if (wait_for_query_response(frontend, MASTER(backend), string, MAJOR(backend)) != POOL_CONTINUE)
 			{
 				/* Cancel current transaction */
 				CancelPacket cancel_packet;
@@ -572,7 +578,7 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 			if (!VALID_BACKEND(i) || IS_MASTER_NODE_ID(i))
 				continue;
 
-			if (wait_for_query_response(frontend, CONNECTION(backend, i), string) != POOL_CONTINUE)
+			if (wait_for_query_response(frontend, CONNECTION(backend, i), string, MAJOR(backend)) != POOL_CONTINUE)
 			{
 				/* Cancel current transaction */
 				CancelPacket cancel_packet;
@@ -592,7 +598,7 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 			if (send_simplequery_message(MASTER(backend), len, string, MAJOR(backend)) != POOL_CONTINUE)
 				return POOL_END;
 
-			if (wait_for_query_response(frontend, MASTER(backend), string) != POOL_CONTINUE)
+			if (wait_for_query_response(frontend, MASTER(backend), string, MAJOR(backend)) != POOL_CONTINUE)
 			{
 				/* Cancel current transaction */
 				CancelPacket cancel_packet;
@@ -615,7 +621,7 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 		if (send_simplequery_message(MASTER(backend), len, string, MAJOR(backend)) != POOL_CONTINUE)
 			return POOL_END;
 
-		if (wait_for_query_response(frontend, MASTER(backend), string) != POOL_CONTINUE)
+		if (wait_for_query_response(frontend, MASTER(backend), string, MAJOR(backend)) != POOL_CONTINUE)
 		{
 				/* Cancel current transaction */
 				CancelPacket cancel_packet;
@@ -635,7 +641,7 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 /*
  * process EXECUTE (V3 only)
  */
-POOL_STATUS Execute(POOL_CONNECTION *frontend, 
+POOL_STATUS Execute(POOL_CONNECTION *frontend,
 						   POOL_CONNECTION_POOL *backend)
 {
 	char *string;		/* portal name + null terminate + max_tobe_returned_rows */
@@ -669,22 +675,30 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 		p_stmt = (PrepareStmt *)portal->stmt;
 
 		string1 = portal->sql_string;
+		pool_debug("Execute: query: %s", string1);
 		node = (Node *)p_stmt->query;
 		strncpy(query_string_buffer, string1, sizeof(query_string_buffer));
 
-		if ((IsA(node, PrepareStmt) || IsA(node, DeallocateStmt) ||
-			 IsA(node, VariableSetStmt)) &&
-			MASTER_SLAVE && TSTATE(backend) != 'E')
+ 		if ((IsA(node, PrepareStmt) || IsA(node, DeallocateStmt) ||
+ 			 IsA(node, VariableSetStmt)) &&
+  			MASTER_SLAVE && TSTATE(backend) != 'E')
 		{
+			/*
+			 * PREPARE, DEALLOCATE, SET, DISCARD
+			 * should be executed on all nodes.  So we set
+			 * force_replication.
+			 */
 			force_replication = 1;
 		}
 		/*
-		 * JDBC driver sends "BEGIN" query internally if setAutoCommit(false).
-		 * But it does not send Sync message after "BEGIN" query.
-		 * In extended query protocol, PostgreSQL returns
-		 * ReadyForQuery when a client sends Sync message.
-		 * We can't know a transaction state...
-		 * So pgpool send Sync message internally.
+		 * JDBC driver sends "BEGIN" query internally if
+		 * setAutoCommit(false).  But it does not send Sync message
+		 * after "BEGIN" query.  In extended query protocol,
+		 * PostgreSQL returns ReadyForQuery when a client sends Sync
+		 * message.  Problem is, pgpool can't know the transaction
+		 * state without receiving ReadyForQuery. So we remember that
+		 * we need to send Sync message internally afterward, whenever
+		 * we receive BEGIN in extended protocol.
 		 */
 		else if (IsA(node, TransactionStmt) && MASTER_SLAVE)
 		{
@@ -692,7 +706,8 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 
 			if (stmt->kind == TRANS_STMT_BEGIN ||
 				stmt->kind == TRANS_STMT_START)
-				receive_extended_begin = true;
+				/* Remember we need to send sync later in extended protocol */
+				receive_extended_begin = 1;
 		}
 
 		if (load_balance_enabled(backend, node, string1))
@@ -744,7 +759,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 			if (send_execute_message(backend, MASTER_NODE_ID, len, string) != POOL_CONTINUE)
 				return POOL_END;
 
-			if (wait_for_query_response(frontend, MASTER(backend), string) != POOL_CONTINUE)
+			if (wait_for_query_response(frontend, MASTER(backend), string, MAJOR(backend)) != POOL_CONTINUE)
 			{
 				/* Cancel current transaction */
 				CancelPacket cancel_packet;
@@ -792,7 +807,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 			if (!VALID_BACKEND(i) || IS_MASTER_NODE_ID(i))
 				continue;
 
-			if (wait_for_query_response(frontend, CONNECTION(backend, i), string) != POOL_CONTINUE)
+			if (wait_for_query_response(frontend, CONNECTION(backend, i), string, MAJOR(backend)) != POOL_CONTINUE)
 			{
 				/* Cancel current transaction */
 				CancelPacket cancel_packet;
@@ -812,7 +827,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 			if (send_execute_message(backend, MASTER_NODE_ID, len, string) != POOL_CONTINUE)
 				return POOL_END;
 
-			if (wait_for_query_response(frontend, MASTER(backend), string) != POOL_CONTINUE)
+			if (wait_for_query_response(frontend, MASTER(backend), string, MAJOR(backend)) != POOL_CONTINUE)
 			{
 				/* Cancel current transaction */
 				CancelPacket cancel_packet;
@@ -831,7 +846,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 		if (send_execute_message(backend, MASTER_NODE_ID, len, string) != POOL_CONTINUE)
 			return POOL_END;
 
-		if (wait_for_query_response(frontend, MASTER(backend), string) != POOL_CONTINUE)
+		if (wait_for_query_response(frontend, MASTER(backend), string, MAJOR(backend)) != POOL_CONTINUE)
 		{
 				/* Cancel current transaction */
 				CancelPacket cancel_packet;
@@ -871,7 +886,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 /*
  * process Parse (V3 only)
  */
-POOL_STATUS Parse(POOL_CONNECTION *frontend, 
+POOL_STATUS Parse(POOL_CONNECTION *frontend,
 						 POOL_CONNECTION_POOL *backend)
 {
 	char kind;
@@ -907,6 +922,9 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 	}
 	else
 	{
+		/* Save last query string for logging purpose */
+		snprintf(query_string_buffer, sizeof(query_string_buffer), "Parse: %s", stmt);
+
 		node = (Node *) lfirst(list_head(parse_tree_list));
 
 		insert_stmt_with_lock = need_insert_lock(backend, stmt, node);
@@ -971,12 +989,12 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 			}
 
 			if (is_strict_query(node))
-				start_internal_transaction(backend, node);
+				start_internal_transaction(frontend, backend, node);
 
 			if (insert_stmt_with_lock)
 			{
 				/* start a transaction if needed and lock the table */
-				status = insert_lock(backend, stmt, (InsertStmt *)node);
+				status = insert_lock(frontend, backend, stmt, (InsertStmt *)node);
 				if (status != POOL_CONTINUE)
 				{
 					return status;
@@ -993,12 +1011,22 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 
 	if (REPLICATION || PARALLEL_MODE || MASTER_SLAVE)
 	{
-		/* We must synchronize because Parse message acquires table
+		/*
+		 * We must synchronize because Parse message acquires table
 		 * locks.
 		 */
-		pool_debug("waiting for master completing the query");
-		if (synchronize(MASTER(backend)))
+		pool_debug("Parse: waiting for master completing the query");
+		if (wait_for_query_response(frontend, MASTER(backend), string, MAJOR(backend)) != POOL_CONTINUE)
+		{
+			/* Cancel current transaction */
+			CancelPacket cancel_packet;
+
+			cancel_packet.protoVersion = htonl(PROTO_CANCEL);
+			cancel_packet.pid = MASTER_CONNECTION(backend)->pid;
+			cancel_packet.key= MASTER_CONNECTION(backend)->key;
+			cancel_request(&cancel_packet);
 			return POOL_END;
+		}
 
 		/*
 		 * We must check deadlock error because a aborted transaction
@@ -1036,9 +1064,18 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 			if (!VALID_BACKEND(i) || IS_MASTER_NODE_ID(i))
 				continue;
 
-			pool_debug("waiting for %dth backend completing the query", i);
-			if (synchronize(CONNECTION(backend, i)))
+			pool_debug("Parse: waiting for %dth backend completing the query", i);
+			if (wait_for_query_response(frontend, CONNECTION(backend, i), string, MAJOR(backend)) != POOL_CONTINUE)
+			{
+				/* Cancel current transaction */
+				CancelPacket cancel_packet;
+
+				cancel_packet.protoVersion = htonl(PROTO_CANCEL);
+				cancel_packet.pid = MASTER_CONNECTION(backend)->pid;
+				cancel_packet.key= MASTER_CONNECTION(backend)->key;
+				cancel_request(&cancel_packet);
 				return POOL_END;
+			}
 		}
 	}
 
@@ -1049,7 +1086,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 
 		if (ret != POOL_CONTINUE)
 			return ret;
-			
+
 		SimpleForwardToFrontend(kind, frontend, backend);
 		if (pool_flush(frontend) < 0)
 			return POOL_ERROR;
@@ -1068,7 +1105,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
  *	 to all DB nodes to abort transaction.
  * - internal transaction is closed
  */
-POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend, 
+POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 								 POOL_CONNECTION_POOL *backend, int send_ready)
 {
 	StartupPacket *sp;
@@ -1137,7 +1174,7 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 		mismatch_ntuples = 0;
 	}
 
-	/* 
+	/*
 	 * if a transaction is started for insert lock, we need to close
 	 * the transaction.
 	 */
@@ -1163,7 +1200,7 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 			pool_debug("ReadyForQuery: transaction state: %c", state);
 		}
 
-		if (end_internal_transaction(backend) != POOL_CONTINUE)
+		if (end_internal_transaction(frontend, backend) != POOL_CONTINUE)
 			return POOL_ERROR;
 	}
 
@@ -1191,7 +1228,7 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 			{
 				if (!VALID_BACKEND(i) || IS_MASTER_NODE_ID(i))
 					continue;
-				
+
 				if (pool_read(CONNECTION(backend, i), &kind1, sizeof(kind)))
 					return POOL_END;
 			}
@@ -1256,10 +1293,10 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 
 	sp = MASTER_CONNECTION(backend)->sp;
 	if (MASTER(backend)->tstate == 'T')
-		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle in transaction", 
+		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle in transaction",
 				 sp->user, sp->database, remote_ps_data);
 	else
-		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle", 
+		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle",
 				 sp->user, sp->database, remote_ps_data);
 	set_ps_display(psbuf, false);
 
@@ -1267,7 +1304,7 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 }
 
 
-POOL_STATUS FunctionCall(POOL_CONNECTION *frontend, 
+POOL_STATUS FunctionCall(POOL_CONNECTION *frontend,
 								POOL_CONNECTION_POOL *backend)
 {
 	char dummy[2];
@@ -1364,7 +1401,7 @@ POOL_STATUS FunctionCall(POOL_CONNECTION *frontend,
 	return POOL_CONTINUE;
 }
 
-POOL_STATUS FunctionResultResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS FunctionResultResponse(POOL_CONNECTION *frontend,
 										  POOL_CONNECTION_POOL *backend)
 {
 	char dummy;
@@ -1426,7 +1463,7 @@ POOL_STATUS FunctionResultResponse(POOL_CONNECTION *frontend,
 	return pool_flush(frontend);
 }
 
-POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 										   POOL_CONNECTION_POOL *backend)
 {
 	char fkind;
@@ -1445,6 +1482,10 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 
 	pool_debug("read kind from frontend %c(%02x)", fkind, fkind);
 
+	/*
+	 * If we have received BEGIN in extended protocol before, we need
+	 * to send a sync message to know the transaction stare.
+	 */
 	if (receive_extended_begin)
 	{
 		receive_extended_begin = 0;
@@ -1486,7 +1527,7 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 			status = Parse(frontend, backend);
 			break;
 
-		case 'S':
+		case 'S':  /* Sync message */
 			receive_extended_begin = 0;
 			/* fall through */
 
@@ -1501,7 +1542,7 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 					MASTER_SLAVE = 0;
 					master_slave_dml = 1;
 				}
-
+	
 				status = SimpleForwardToBackend(fkind, frontend, backend);
 				for (i=0;i<NUM_BACKENDS;i++)
 				{
@@ -1527,7 +1568,7 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 	return status;
 }
 
-POOL_STATUS CompleteCommandResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS CompleteCommandResponse(POOL_CONNECTION *frontend,
 										   POOL_CONNECTION_POOL *backend)
 {
 	int i;
@@ -1556,7 +1597,7 @@ POOL_STATUS CompleteCommandResponse(POOL_CONNECTION *frontend,
 		{
 			pool_debug("Complete Command Response: message length does not match between master(%d \"%s\",) and %d th server (%d \"%s\",)",
 					   len, string, i, len1, string1);
-			
+
 			free(string1);
 			return POOL_END;
 		}
@@ -1574,7 +1615,7 @@ POOL_STATUS CompleteCommandResponse(POOL_CONNECTION *frontend,
 	return pool_flush(frontend);
 }
 
-int RowDescription(POOL_CONNECTION *frontend, 
+int RowDescription(POOL_CONNECTION *frontend,
 						  POOL_CONNECTION_POOL *backend,
 						  short *result)
 {
@@ -1701,7 +1742,7 @@ int RowDescription(POOL_CONNECTION *frontend,
 	return pool_flush(frontend);
 }
 
-POOL_STATUS AsciiRow(POOL_CONNECTION *frontend, 
+POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 							POOL_CONNECTION_POOL *backend,
 							short num_fields)
 {
@@ -1813,7 +1854,7 @@ POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 	return POOL_CONTINUE;
 }
 
-POOL_STATUS BinaryRow(POOL_CONNECTION *frontend, 
+POOL_STATUS BinaryRow(POOL_CONNECTION *frontend,
 							 POOL_CONNECTION_POOL *backend,
 							 short num_fields)
 {
@@ -1865,14 +1906,14 @@ POOL_STATUS BinaryRow(POOL_CONNECTION *frontend,
 		{
 			/* field size */
 			if (pool_read(MASTER(backend), &size, sizeof(int)) < 0)
-				return POOL_END;			
+				return POOL_END;
 			for (j=0;j<NUM_BACKENDS;j++)
 			{
 				if (VALID_BACKEND(j) && !IS_MASTER_NODE_ID(j))
 				{
 					/* field size */
 					if (pool_read(CONNECTION(backend, i), &size, sizeof(int)) < 0)
-						return POOL_END;			
+						return POOL_END;
 
 					/* XXX: field size maybe different among
 					   backends. If we were a paranoid, we have to treat
@@ -1915,7 +1956,7 @@ POOL_STATUS BinaryRow(POOL_CONNECTION *frontend,
 	return POOL_CONTINUE;
 }
 
-POOL_STATUS CursorResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS CursorResponse(POOL_CONNECTION *frontend,
 								  POOL_CONNECTION_POOL *backend)
 {
 	char *string = NULL;
@@ -1964,7 +2005,7 @@ POOL_STATUS CursorResponse(POOL_CONNECTION *frontend,
 	return POOL_CONTINUE;
 }
 
-POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend,
 						  POOL_CONNECTION_POOL *backend)
 {
 	char *string = NULL;
@@ -1992,11 +2033,11 @@ POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend,
 		TSTATE(backend) = 'E';
 	else
 		TSTATE(backend) = 'I';
-			
+
 	return POOL_CONTINUE;
 }
 
-POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend,
 								  POOL_CONNECTION_POOL *backend)
 {
 	char *string = NULL;
@@ -2023,7 +2064,7 @@ POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend,
 	return POOL_CONTINUE;
 }
 
-POOL_STATUS CopyInResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS CopyInResponse(POOL_CONNECTION *frontend,
 								  POOL_CONNECTION_POOL *backend)
 {
 	POOL_STATUS status;
@@ -2044,7 +2085,7 @@ POOL_STATUS CopyInResponse(POOL_CONNECTION *frontend,
 	return status;
 }
 
-POOL_STATUS CopyOutResponse(POOL_CONNECTION *frontend, 
+POOL_STATUS CopyOutResponse(POOL_CONNECTION *frontend,
 								   POOL_CONNECTION_POOL *backend)
 {
 	POOL_STATUS status;
@@ -2234,7 +2275,7 @@ POOL_STATUS CopyDataRows(POOL_CONNECTION *frontend,
 			}
 		}
 		else
-			pool_write(frontend, string, len);		
+			pool_write(frontend, string, len);
 
 		if (len == PROTO_MAJOR_V3)
 		{
@@ -2331,7 +2372,7 @@ static int check_errors(POOL_CONNECTION_POOL *backend, int backend_id)
 	/*
 	 * check "SET TRANSACTION ISOLATION LEVEL must be called before any query" error.
 	 * This happens in following scenario:
-	 * 
+	 *
 	 * M:S1:BEGIN;
 	 * S:S1:BEGIN;
 	 * M:S1:SELECT 1; <-- only sent to MASTER
