@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/recovery.c,v 1.4 2007/11/21 04:47:09 y-asaba Exp $
- * 
- * pgpool: a language independent connection pool server for PostgreSQL 
+ * $Header: /cvsroot/pgpool/pgpool-II/recovery.c,v 1.12.2.1 2009/08/22 04:19:49 t-ishii Exp $
+ *
+ * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2007	PgPool Global Development Group
+ * Copyright (c) 2003-2008	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -30,10 +30,13 @@
 #include "pool.h"
 #include "libpq-fe.h"
 
-#define WAIT_RETRY_COUNT 30
+#define WAIT_RETRY_COUNT (pool_config->recovery_timeout / 3)
+
+#define FIRST_STAGE 0
+#define SECOND_STAGE 1
 
 static int exec_checkpoint(PGconn *conn);
-static int exec_recovery(PGconn *conn, BackendInfo *backend, char first_stage);
+static int exec_recovery(PGconn *conn, BackendInfo *backend, char stage);
 static int exec_remote_start(PGconn *conn, BackendInfo *backend);
 static PGconn *connect_backend_libpq(BackendInfo *backend);
 static int wait_connection_closed(void);
@@ -41,12 +44,16 @@ static int check_postmaster_started(BackendInfo *backend);
 
 static char recovery_command[1024];
 
+extern volatile sig_atomic_t pcp_wakeup_request;
+
 int start_recovery(int recovery_node)
 {
 	BackendInfo *backend;
 	BackendInfo *recovery_backend;
 	PGconn *conn;
 
+	pool_log("starting recovering node %d", recovery_node);
+
 	if (VALID_BACKEND(recovery_node))
 	{
 		pool_error("start_recovery: backend node %d is alive", recovery_node);
@@ -73,12 +80,19 @@ int start_recovery(int recovery_node)
 		pool_error("start_recovery: CHECKPOINT failed");
 		return 1;
 	}
-	if (exec_recovery(conn, recovery_backend, 1) != 0)
+
+	pool_log("CHECKPOINT in the 1st stage done");
+
+	if (exec_recovery(conn, recovery_backend, FIRST_STAGE) != 0)
 	{
 		PQfinish(conn);
 		return 1;
 	}
 
+	pool_log("1st stage is done");
+
+	pool_log("starting 2nd stage");
+
 	/* 2nd stage */
 	*InRecovery = 1;
 	if (wait_connection_closed() != 0)
@@ -88,13 +102,18 @@ int start_recovery(int recovery_node)
 		return 1;
 	}
 
+	pool_log("all connections from clients have been closed");
+
 	if (exec_checkpoint(conn) != 0)
 	{
 		PQfinish(conn);
 		pool_error("start_recovery: CHECKPOINT failed");
 		return 1;
 	}
-	if (exec_recovery(conn, recovery_backend, 0) != 0)
+
+	pool_log("CHECKPOINT in the 2nd stage done");
+
+	if (exec_recovery(conn, recovery_backend, SECOND_STAGE) != 0)
 	{
 		PQfinish(conn);
 		return 1;
@@ -113,11 +132,30 @@ int start_recovery(int recovery_node)
 		return 1;
 	}
 
+	pool_log("%d node restarted", recovery_node);
+
+	/*
+	 * reset failover completion flag.  this is necessary since
+	 * previous failover/failback will set the flag to 1.
+	 */
+	pcp_wakeup_request = 0;
+
+	/* send failback request to pgpool parent */
 	send_failback_request(recovery_node);
-	pause(); /* wait for failback */
+
+	/* wait for failback */
+	while (!pcp_wakeup_request)
+	{
+		struct timeval t = {1, 0};
+		/* polling SIGUSR2 signal every 1 sec */
+		select(0, NULL, NULL, NULL, &t);
+	}
+	pcp_wakeup_request = 0;
 
 	PQfinish(conn);
 
+	pool_log("recovery done");
+
 	return 0;
 }
 
@@ -149,7 +187,7 @@ static int exec_checkpoint(PGconn *conn)
 /*
  * Call pgpool_recovery() function.
  */
-static int exec_recovery(PGconn *conn, BackendInfo *backend, char first_stage)
+static int exec_recovery(PGconn *conn, BackendInfo *backend, char stage)
 {
 	PGresult *result;
 	char *hostname;
@@ -161,7 +199,8 @@ static int exec_recovery(PGconn *conn, BackendInfo *backend, char first_stage)
 	else
 		hostname = backend->backend_hostname;
 
-	script = first_stage ? pool_config->recovery_1st_stage_command : pool_config->recovery_2nd_stage_command;
+	script = (stage == FIRST_STAGE) ?
+		pool_config->recovery_1st_stage_command : pool_config->recovery_2nd_stage_command;
 
 	if (script == NULL || strlen(script) == 0)
 	{
@@ -176,11 +215,17 @@ static int exec_recovery(PGconn *conn, BackendInfo *backend, char first_stage)
 			 hostname,
 			 backend->backend_data_directory);
 
+	pool_log("starting recovery command: \"%s\"", recovery_command);
+
 	pool_debug("exec_recovery: start recovery");
 	result = PQexec(conn, recovery_command);
 	r = (PQresultStatus(result) !=  PGRES_TUPLES_OK);
 	if (r != 0)
-		pool_error("exec_recovery: pgpool_recovery failed: %s", PQresultErrorMessage(result));
+	{
+		pool_error("exec_recovery: %s command failed at %s",
+				   script,
+				   (stage == FIRST_STAGE) ? "1st stage" : "2nd stage");
+	}
 	PQclear(result);
 	pool_debug("exec_recovery: finish recovery");
 	return r;
@@ -220,14 +265,13 @@ static int exec_remote_start(PGconn *conn, BackendInfo *backend)
  */
 static int check_postmaster_started(BackendInfo *backend)
 {
-	int i;
+	int i = 0;
 	char port_str[16];
 	PGconn *conn;
 
 	snprintf(port_str, sizeof(port_str),
 			 "%d", backend->backend_port);
-	for (i = 0; i < WAIT_RETRY_COUNT; i++)
-	{
+	do {
 		ConnStatusType r;
 		conn = PQsetdbLogin(backend->backend_hostname,
 							port_str,
@@ -241,8 +285,11 @@ static int check_postmaster_started(BackendInfo *backend)
 		if (r == CONNECTION_OK)
 			return 0;
 
-		sleep(3);
-	}
+		if (WAIT_RETRY_COUNT != 0)
+			sleep(3);
+	} while (i++ < WAIT_RETRY_COUNT);
+
+	pool_error("check_postmaster_started: remote host start up did not finish in %d sec.", pool_config->recovery_timeout);
 	return 1;
 }
 
@@ -271,18 +318,20 @@ static PGconn *connect_backend_libpq(BackendInfo *backend)
 
 /*
  * Wait all connections are closed.
- * If connection remained, sleep 3 sec and retry to wait.
  */
 static int wait_connection_closed(void)
 {
-	int i;
+	int i = 0;
+
+	do {
 
-	for (i = 0; i < WAIT_RETRY_COUNT; i++)
-	{
 		if (Req_info->conn_counter == 0)
 			return 0;
-		sleep(3);
-	}
 
+		if (WAIT_RETRY_COUNT != 0)
+			sleep(3);
+	} while (i++ < WAIT_RETRY_COUNT);
+
+	pool_error("wait_connection_closed: existing connections did not close in %d sec.", pool_config->recovery_timeout);
 	return 1;
 }
